@@ -1,5 +1,5 @@
 ## color_util.R  |  unikn
-## spds | uni.kn | 2022 08 13
+## spds | uni.kn | 2022 09 13
 ## ---------------------------
 
 ## Utility functions for converting colors, 
@@ -12,7 +12,7 @@
 # col2rgb in grDevices: ------ 
 
 ## Check: 
-# col2rgb("black", alpha = FALSE)
+# col2rgb("black", alpha = FALSE)  # Note: alpha is Boolean argument.
 # col2rgb("black", alpha = TRUE)
 # col2rgb("black")
 
@@ -25,7 +25,9 @@
 # get_alpha: Get color transparency / alpha values: ------
 
 get_alpha <- function(pal){
+  
   grDevices::col2rgb(pal, alpha = TRUE)["alpha", ] 
+  
 } # get_alpha().
 
 ## Check:
@@ -38,7 +40,9 @@ get_alpha <- function(pal){
 # rgb2hex color conversion function: ------ 
 
 rgb2hex <- function(R, G, B) {
+  
   rgb(R, G, B, maxColorValue = 255)
+  
 } # rgb2hex().
 
 ## Check:
@@ -49,58 +53,84 @@ rgb2hex <- function(R, G, B) {
 
 # col2hex color conversion function: ------ 
 
-col2hex <- function(col, alpha = alpha) {
-  rgb(t(col2rgb(col)), alpha = alpha, maxColorValue = 255)
+col2hex <- function(col, alpha = NA, use_alpha = FALSE) {
+  
+  if (use_alpha){
+    
+    if (is.na(alpha)){ # Only get alpha values of col:
+      
+      rgb(t(col2rgb(col)), alpha = get_alpha(col), maxColorValue = 255)
+      
+    } else { # Use the alpha value(s) provided:
+      
+      # ToDo: verify that alpha is numeric and in [0, 1]
+      
+      rgb(t(col2rgb(col)), alpha = alpha * 255, maxColorValue = 255)
+      
+    }
+    
+  } else { # default: 
+    
+    rgb(t(col2rgb(col)), maxColorValue = 255)
+    
+  }
+  
 } # col2hex().
 
 ## Check: 
-# hex1 <- col2hex("black", alpha = 255/2)
-# hex2 <- col2hex("white", alpha = 255/2)
-# hex3 <- col2hex("gold", alpha = 255/2)
-# hex4 <- col2hex("steelblue", alpha = 255/2)
+# hex1 <- col2hex("black", alpha = 1/2, use_alpha = TRUE)
+# hex2 <- col2hex("white", alpha = 2/3, use_alpha = TRUE)
+# hex3 <- col2hex("gold", alpha = 1/3, use_alpha = TRUE)
+# hex4 <- col2hex("steelblue", alpha = 0, use_alpha = FALSE)
 # seecol(pal = c(hex1, hex2, hex3, hex4), n = "all")
 # 
 # # Note 2 limitations: 
 # 1. As col2hex assumes a SCALAR alpha value,
 #    the following does currently NOT work:
 # col2hex("black", alpha = c(1/4, 1/2, 3/4))
-# # ToDo 1: Vectorize col2hex arguments.
+# # ToDo: Vectorize col2hex arguments?
 #
-# 2. Current col2hex does not detect/use transparency differences: 
+# # 2. By default, col2hex does not detect/use transparency differences: 
+# tblack <- ac("black", alpha = seq(0, 1, by = 0.25))
+# # seecol(tblack)
 # col2hex(tblack)  # ignores transparency!
-# # ToDo 2: Allow retaining transparency in col2hex:
-# col2hex(tblack, alpha = alpha(tblack))
-# # (and consider using get_alpha() for this).
+# # Use color transparency in col2hex:
+# col2hex(tblack, use_alpha = TRUE) # uses alpha values without setting them
+# # seecol(col2hex(tblack, use_alpha = TRUE))
 
 
 
-# isHexCol: Helper function to detect HEX-colors: ------ 
+# is_hex_col: Helper function to detect HEX-colors: ------ 
 
-isHexCol <- function(color) {
+is_hex_col <- function(color) {
+  
   return(grepl(pattern = "^#[0-9A-Fa-f]{6,}", color))
-} # isHexCol().
+  
+} # is_hex_col().
 
 ## Check:
-# isHexCol("black")
-# isHexCol(col2hex("black"))
-# isHexCol(rgb2hex(0, 0, 0))
+# is_hex_col("black")
+# is_hex_col(col2hex("black"))
+# is_hex_col(rgb2hex(0, 0, 0))
 
 
 
-# isCol: Helper function to detect any color (in an individual character string): ------ 
+# is_col: Helper function to detect any color (in an individual character string): ------ 
 
-isCol <- function(color) {
-  return(isHexCol(color) | color %in% colors())
-} # isCol(). 
+is_col <- function(color) {
+  
+  return(is_hex_col(color) | color %in% colors())
+  
+} # is_col(). 
 
 ## Check:
-# isCol("white")
-# isCol(col2hex("black", alpha = 255/2))
-# isCol(NA)
-# isCol("bumblebee")
+# is_col("white")
+# is_col(col2hex("black", alpha = 255/2))
+# is_col(NA)
+# is_col("bumblebee")
 
 # BUT note: 
-# isCol(col2rgb("white"))  # => FALSE FALSE FALSE
+# is_col(col2rgb("white"))  # => FALSE FALSE FALSE
 
 
 
@@ -155,65 +185,107 @@ col_distance <- function(col_1, col_2){
 #   irrespective of their names/values).
 # - Color transparency is only considered when use_alpha = TRUE.
 # - grDevices::colors() has a 'distinct = TRUE' argument to remove visual duplicates.
+# - By default, visually distinct colors (i.e., with different color values) but same NAME are retained, 
+#   but setting use_names = TRUE removes colors with duplicate names (even if their color values differ).
 
-col_distinct <- function(pal, use_alpha = FALSE){
+col_distinct <- function(pal, use_hex = TRUE, use_alpha = FALSE, use_names = FALSE){
   
-  # Prepare: ---- 
+  # Prepare: ------ 
   
-  if (any(isCol(pal) == FALSE)){
+  if (any(is_col(pal) == FALSE)){
     stop("pal contains non-colors")
   }
   
+  # Initialize:
+  hex_dupes <- FALSE   # default: all different HEX values
+  alpha_dupes <- TRUE  # default: all same alpha values
+  name_dupes <- FALSE  # default: no names/all different names
   
-  # Main: ----
+  out_pal <- NA
+  
+  
+  # Main: ------
+  
+  # 1. Color HEX values: ---- 
+  
+  if (use_hex){
+    
+    pal_hex <- col2hex(pal)  # ignores transparency!
+    hex_dupes <- duplicated(pal_hex)  # logical ix of duplicated hex values
+    
+  } 
+  
+  
+  # 2. alpha values: ----
   
-  # 1. alpha values:
   if (use_alpha){
     
     # Get transparency values:
     pal_alpha <- get_alpha(pal)
     
-    ix_alpha_dupes <- duplicated(pal_alpha)  # duplicate alpha values
+    alpha_dupes <- duplicated(pal_alpha)  # logical ix of duplicate alpha values
     
-  } else {
+  } 
+  
+  # Apply indices 1+2: ----
+  # print(hex_dupes)   # 4debugging
+  # print(alpha_dupes) # 4debugging
+  
+  cur_pal <- pal[(hex_dupes & alpha_dupes) == FALSE]
+  
+  
+  # 3. Color names (for cur_pal only): ---- 
+  
+  if ((is.null(names(cur_pal)) == FALSE) & use_names){
     
-    ix_alpha_dupes <- TRUE  # default: all same alpha values
+    pal_names <- names(cur_pal)
+    name_dupes <- duplicated(pal_names)
     
   }
   
-  # 2. HEX values:
-  pal_hex <- col2hex(pal)  # ignores transparency!
+  # Apply index 3: ---- 
+  # print(name_dupes) # 4debugging
   
-  ix_hex_dupes <- duplicated(pal_hex)  # duplicated hex values
+  if (use_names){
+    out_pal <- cur_pal[name_dupes == FALSE]
+  } else {
+    out_pal <- cur_pal
+  }
   
   
-  # Combine logical indices:
-  ix_pal_dupes <- ix_hex_dupes & ix_alpha_dupes
+  # Output: ------
   
-  
-  # Output: ----
-  
-  # Return pal without duplicates: 
-  pal[ix_pal_dupes == FALSE]
+  return(out_pal)
   
 } # col_distinct().
 
 
-## Check:
-# p1 <- c("gray", "grey", "black", "grey0", "red", "red1", "red2", "red3")
+# ## Check:
+# (p1 <- c("gray", "grey", "black", "grey0", rgb(0, 0, 0), "red", "red1", rgb(1, 0, 0), "red2", "red3"))
+# seecol(p1)
 # col_distinct(p1)
+# col_distinct(p1, use_hex = FALSE, use_alpha = TRUE, use_names = TRUE)
 # 
-# (p2 <- usecol(c(pal_unikn, pal_seeblau)))
+# # with repeated color values (in custom pals):
+# (p2 <- usecol(c("black", pal_unikn, pal_seeblau, pal_grau, "white")))
 # col_distinct(p2)
+# seecol(col_distinct(p2))
 # 
-# # By default, color transparency is being ignored:
+# # use_names:
+# (p3 <- newpal(col = c("black", "gray", "grey", "white"), names = c("extreme", "med", "med", "extreme")))
+# col_distinct(p3)  # removes duplicate color values, but keeps colors with duplicate names
+# col_distinct(p3, use_names = TRUE)  # removes duplicate names (despite distinct colors)
+# col_distinct(p3, use_hex = FALSE, use_names = TRUE)
+# 
+# # use_alpha: By default, color transparency is being ignored:
 # tblack <- ac("black", alpha = seq(0, 1, by = 0.25))
 # seecol(tblack)        # Shows 5 shades, but
 # col_distinct(tblack)  # all share same basic color!
-# col_distinct(tblack, use_alpha = TRUE)  # USES color transparency!
+# col_distinct(tblack, use_alpha = TRUE)  # USES color transparency (to keep visual different shades of same color)
+# col_distinct(tblack, use_hex = FALSE, use_names = TRUE)
 # 
 # # Invalid inputs:
-# col_distinct(c("black", "NOT a color", "white"))
+# col_distinct(c("black", "NO color", "white"))
 
 
 
@@ -231,7 +303,7 @@ parse_pal <- function(pal) {
   # Check if pal is legible (already a color palette): 
   vector_input <- tryCatch(
     {
-      all(sapply(pal, isCol))
+      all(sapply(pal, is_col))
     },
     
     error = function(e) {
@@ -299,7 +371,7 @@ parse_pal <- function(pal) {
     if ( any(!elemex) ) { # only if not all inputs have been resolved
       
       # Those which are still unknown: Are those colors? 
-      elemex[!elemex] <- sapply(elem[!elemex], isCol)
+      elemex[!elemex] <- sapply(elem[!elemex], is_col)
       
     }
     
@@ -331,8 +403,10 @@ parse_pal <- function(pal) {
       
     }
     
-    # Get all palettes: 
-    out <- lapply(elem, function(x) if( isCol(x) ) x else get(x) )
+    
+    # Get all palettes: ---- 
+    out <- lapply(elem, function(x) if (is_col(x)) {x} else {get(x)} )
+    
     
     # Apply any previously detected functions: ----  
     if ( any(!is.na(funs)) ) {
@@ -343,19 +417,19 @@ parse_pal <- function(pal) {
       
     }
     
-    # Create the output: 
+    # Create the output: ---- 
     out <- unname(out)  # finish the palette by removing upper level (palette) names.
     
   }
   
+  # Output: ---- 
+  
   out <- unlist(out)
   
   # Provide missing names, by using the color:
   ix_nameless <- is.null(names(out)) | names(out) == ""
   names(out)[ix_nameless] <- out[ix_nameless]
   
-  # Output: ---- 
-  
   return(out)
   
 } # parse_pal().
@@ -383,7 +457,7 @@ getpal_key <- function(pal = "all", n = "all", alpha = NA) {
   } else {
     
     if ( pal %in% keys[1:3] )   key <- "all"
-    if ( pal %in% keys [4:6] )  key <- "basic"
+    if ( pal %in% keys[4:6] )   key <- "basic"
     if ( pal %in% keys[7:9] )   key <- "pair"
     if ( pal %in% keys[10:12] ) key <- "pref"
     if ( pal %in% keys[13:15] ) key <- "grad"
@@ -412,7 +486,7 @@ getpal_key <- function(pal = "all", n = "all", alpha = NA) {
       if ( !typeof(x) %in% c("vector", "list") ) {
         is_color <- FALSE
       } else {
-        is_color <- isHexCol(color = x)
+        is_color <- is_hex_col(color = x)
       }
       return(all(is_color))  # are all entries colors?
       
@@ -455,11 +529,9 @@ getpal_key <- function(pal = "all", n = "all", alpha = NA) {
 
 
 
-
 ## 3. Plotting functions: ------
 
 
-
 # plot_shape: Plot a shape in a certain color: ------
 
 plot_shape <- function(pos_x, pos_y,  # midpoint of shape  
@@ -548,28 +620,31 @@ plot_col <- function(x,         # a *vector* of colors to be plotted.
     if (dev.cur() == 1) {
       stop("No graphic device to be plotted on.  Please open a plot or set plot.new to 'TRUE'.")
     }
+    
   }
   
+  
   # 2. Position parameters: -----
   
   # Shape centers:
-  xpos <- (1:len_x) - 0.5  # Note: Subtracting .5 assumes a shape width of 1.
+  xpos <- (1:len_x) - 0.5  # subtracting 0.5 assumes a shape width of 1.
   
-  # +++ here now +++: Allow scaling shape widths to fill a FIXED total width 
-  #                   (e.g., each shape a width of 10/len_x).
+  # ToDo: Allow scaling shape widths to fill a FIXED total width 
+  #       (e.g., each shape with a width of 10/len_x).
   
   # Adjust xpos by distance:
   mid <- mean(xpos)  # get midpoint. 
-  add <- cumsum(rep(distance, sum(xpos < mid)))  # values to be added to the first half. 
-  sub <- add * (-1)                              # values to be subtracted from the second half. 
-  xpos <- xpos + if(len_x %% 2 == 0) c(rev(sub), add) else  # even numbers: no center position needed.
-    c(rev(sub), 0, add)                                     # odd numbers: include the middle (0). 
+  add <- cumsum(rep(distance, sum(xpos < mid)))  # values to be added to the 1st half 
+  sub <- add * (-1)                              # values to be subtracted from the 2nd half 
+  xpos <- xpos + if (len_x %% 2 == 0) {c(rev(sub), add)} else  # even numbers: no center position needed
+  {c(rev(sub), 0, add)}                                      # odd numbers: include a middle (0)
   
   # Recycle other constants (to len_x):
   ypos <- rep(ypos, length.out = len_x) 
   xlen <- rep(xlen, length.out = len_x)
   ylen <- rep(ylen, length.out = len_x)
   
+  
   # 3. Plot shapes: ------ 
   
   plot_shape(pos_x = xpos,  # x positions of the shapes. 
